BT <- function(x, y)
{
#>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                             <<<<<
#>>>>   Estimation by Bradley-Terry Model         <<<<<
#>>>>                                             <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	m <- length(x)
	Pi <- c(1:m)
	Pi1 <- c(1:m)
	r <- c(1:m)
	PPi <- c(1:m)
	seido <- 1e-05
	k <- 0
	for(i in 1:m) {
		Pi[i] <- 50
		k <- k + Pi[i]
	}
	repeat {
		for(i in 1:m)
			r[i] <- 0
		for(i in 1:m) {
			for(j in 1:m) {
				if(j != i)
				  r[i] <- r[i] + (y[i, j]/(Pi[i] + Pi[j]))
			}
		}
		t <- 0
		for(i in 1:m) {
			PPi[i] <- x[i]/r[i]
			t <- t + PPi[i]
		}
		for(i in 1:m) {
			Pi1[i] <- (k * PPi[i])/t
		}
		d <- 0
		for(i in 1:m)
			d <- d + (Pi[i] - Pi1[i])^2
		if(d < seido^2)
			break
		for(i in 1:m)
			Pi[i] <- Pi1[i]
	}
	Pi #
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#           x: vector of the number of wins (length: n).
#           y: n X n matrix for the number of match between n teams.
#
#  >>> output <<<
#       vector of PIs (TSUYOSA).
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                  03/21/99  by kito, Takada
}

KBT <- function(x, s, k)
{
#>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                             <<<<<
#>>>>   Modified BT Model for home and away       <<<<<
#>>>>                                             <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	m <- length(x)
	kseido <- 1e-07
	w <- t <- r1 <- r2 <- Pi <- Pw <- PPi <- PPw <- Pi1 <- Pw1 <- 1:m
	z <- matrix(0, m, m)
	Pi <- rep(50, m)
	Pw <- rep(1, m)
	t <- w <- rep(0, m)
	kk1 <- sum(Pi)
	for(i in 1:m) {
		for(j in 1:m) {
			t[i] <- t[i] + k[i, j] + s[i, j]
			w[i] <- w[i] + k[i, j] + k[j, i]
			z[i, j] <- k[i, j] + s[j, i]
		}
	}
	repeat {
		r1 <- r2 <- rep(0, m)
		for(i in 1:m) {
			for(j in 1:m) {
				if(j != i) {
				  r1[i] <- r1[i] + (((z[i, j] * Pw[i] * Pw[j])/(
				    Pw[i] * Pw[j] * Pi[i] + Pi[j])) + (z[j, i]/(
				    Pw[i] * Pw[j] * Pi[j] + Pi[i])))
				  r2[i] <- r2[i] + (((z[i, j] * Pi[i] * Pw[j])/(
				    Pw[i] * Pw[j] * Pi[i] + Pi[j])) + ((z[j, i] *
				    Pi[j] * Pw[j])/(Pw[i] * Pw[j] * Pi[j] + Pi[
				    i])))
				}
			}
		}
		for(i in 1:m) {
			PPi[i] <- t[i]/r1[i]
			PPw[i] <- w[i]/r2[i]
		}
		t1 <- sum(PPi)
		for(i in 1:m) {
			Pi1[i] <- (kk1 * PPi[i])/t1
			Pw1[i] <- PPw[i]
		}
		d1 <- d2 <- 0
		for(i in 1:m) {
			d1 <- d1 + (Pi[i] - Pi1[i])^2
			d2 <- d2 + (Pw[i] - Pw1[i])^2
		}
		if((d1 < kseido) && (d2 < kseido))
			break
		for(i in 1:m) {
			Pi[i] <- Pi1[i]
			Pw[i] <- Pw1[i]
		}
	}
	list(tsuyosa = Pi, omega = Pw) #
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#           x: vector of the number of wins (length: n).
#           s: n X n matrix for the number of wins of row team for
#              column team where column team is home.
#           k: n X n matrix for the nubers of wins of row team for
#              column team where row team is home.
#
#  >>> output <<<
#     tsuyosa: vector of PIs (TSUYOSA).
#       omega: vector of home&away coefficients.
#              home's TSUYOSA is PI x Omega,
#              away's TSUYOSA is PI / Omega.
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                  03/21/99  by kito, Takada
}

RBT <- function(x, y, z)
{
#>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                             <<<<<
#>>>>   Skew matrix (KAMO-NIGATE) by BT Model     <<<<<
#>>>>                                             <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	m <- length(x)
	Pi <- Pi1 <- r <- PPi <- 1:m
	seido <- 1e-05
	k <- 0
	Pi <- rep(50, m)
	k <- sum(Pi)
	repeat {
		r <- rep(0, m)
		for(i in 1:m) {
			for(j in 1:m) {
				if(j != i)
				  r[i] <- r[i] + (y[i, j]/(Pi[i] + Pi[j]))
			}
		}
		t <- 0
		for(i in 1:m) {
			PPi[i] <- x[i]/r[i]
		}
		t <- sum(PPi)
		for(i in 1:m) {
			Pi1[i] <- (k * PPi[i])/t
		}
		d <- 0
		for(i in 1:m)
			d <- d + (Pi[i] - Pi1[i])^2
		if(d < seido^2)
			break
		for(i in 1:m)
			Pi[i] <- Pi1[i]
	}
	a <- matrix(0, m, m)
	for(i in 1:m) {
		for(j in 1:m) {
			a[i, j] <- (log10(((z[i, j] + 0.5)/(z[j, i] + 0.5)) * (
				Pi[j]/Pi[i])))/log10(2)
		}
	}
	a #
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#           x: vector of the number of wins (length: n).
#           y: n X n matrix for the number of match between n teams.
#           z: n X n matrix for the nubers of wins of row team for
#              column team.
#
#  >>> output <<<
#       skew matrix (KAMO-NIGATE).
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                  03/21/99  by kito, Takada
}

ZBT <- function(x, y, z)
{
#>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                             <<<<<
#>>>>   Plot of skew matrix (KAMO-NIGATE) by BTM  <<<<<
#>>>>                                             <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	m <- length(x)
	Pi <- Pi1 <- r <- PPi <- 1:m
	seido <- 1e-05
	k <- 0
	Pi <- rep(50, m)
	k <- sum(Pi)
	repeat {
		r <- rep(0, m)
		for(i in 1:m) {
			for(j in 1:m) {
				if(j != i)
				  r[i] <- r[i] + (y[i, j]/(Pi[i] + Pi[j]))
			}
		}
		t <- 0
		for(i in 1:m) {
			PPi[i] <- x[i]/r[i]
		}
		t <- sum(PPi)
		for(i in 1:m) {
			Pi1[i] <- (k * PPi[i])/t
		}
		d <- 0
		for(i in 1:m)
			d <- d + (Pi[i] - Pi1[i])^2
		if(d < seido^2)
			break
		for(i in 1:m)
			Pi[i] <- Pi1[i]
	}
	a <- matrix(0, m, m)
	for(i in 1:m) {
		for(j in 1:m) {
			a[i, j] <- (log10(((z[i, j] + 0.5)/(z[j, i] + 0.5)) * (
				Pi[j]/Pi[i])))/log10(2)
		}
	}
	tokui <- svd(a, m, m)
	tokui
	xz <- yz <- 1:m
	for(i in 1:m) {
		xz[i] <- tokui[["u"]][i, 1]
		yz[i] <- tokui[["u"]][i, 2]
	}
	ram1 <- tokui[["d"]][1]
	for(i in 1:m) {
		xz[i] <- xz[i] * sqrt(ram1)
		yz[i] <- yz[i] * sqrt(ram1)
	}
	plot(xz, yz, type = "n")
	text(xz, yz)
	invisible(xz) #
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#           x: vector of the number of wins (length: n).
#           y: n X n matrix for the number of match between n teams.
#           z: n X n matrix for the nubers of wins of row team for
#              column team.
#           p: plot type (=0(default):plot by character,
#                         =1: plot in the new window).
#
#  >>> output <<<
#       2-dimensional plot for skew matrix (KAMO-NIGATE).
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                           03/21/99  by  kito, Takada
#                        modified for R     11/08/02  by  matsu
#                                           01/30/06  by  matsu
}
