onewayanova<-function(dat)
{
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>          The method of ANOVA (one-way)                <<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  i <- j <- 1
  a <- b <- 0
  m <- n <- N <- NULL
  while(i <= nrow(dat)) {
    while(j <= ncol(dat)) {
      if(is.na(dat[i,j]) == FALSE) {
        b <- b + 1
        a <- dat[i,j]
        n <- append(n, a)
      }
      j <- j + 1
    }
    N <- append(N, n)
    m <- append(m, b)      
    i <- i + 1
    j <- 1
    b <- 0
    n <- NULL
  }
  i <- j<- h <- 1
  S<-c(1:length(N))
  while(h<=length(m) && j!=length(N)){
    while(i<=m[h]){
      S[j]<-h
      i<-i+1
      j<-j+1
    }
    h<-h+1
    i<-1
  }
  y<-c(N)
  x<-factor(S)
  d <- data.frame(y=y, x=x)
  answer<-summary(aov(y~x, d))
  list(answer = answer)
#>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>
#  >>>input<<<
#       dat: matrix data with equal variance of each groups
#            (The data with which sample sizes differ is also possible)
#            (NA is put into missing values)
#
#  >>>output<<<
#       answer        : factor,residuals 
#                       (degrees of freedom,Sum of Squaes, Mean Square,
#                        F-value, F-test result)
#                     : Significance level
#
#>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<
#
#                                                         01/17/06  by  Horiuchi
}

SNK <- function(dat, alpha = 0.05) 
{
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>          The method of Student-Newman-Kuels           <<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  i <- j <- 1
  a <- 0
  n <- NULL
  while(i <= nrow(dat)) {
    while(j <= ncol(dat)) {
      if(is.na(dat[i,j]) == FALSE) {
        a <- a + 1
      }
      j <- j + 1
    }
    n <- append(n, a)
    i <- i + 1
    j <- 1
    a <- 0
  }
  i <- 1
  mean <- u.var <- NULL
  while(i <= length(n)){
     mean <- append(mean,mean(t(dat[i,1:n[i]])))
     u.var <- append(u.var,var(t(dat[i,1:n[i]])))
     i <- i + 1
  }
  phi.e <- sum(n) - length(n)
  squares <- sum((n - 1) * u.var)
  var.e <- squares / phi.e
  t <-t1<- matrix(0, ncol = length(n), nrow = length(n))
  colnames(t)<-colnames(t1) <- paste("Group",1:ncol(t))
  rownames(t)<-rownames(t1) <- paste("Group",1:nrow(t))
  i <- j <- 1
  while(i <= length(n)) {
     while(j <= length(n)) {
        if(j > i){
          t1[i,j]<-t[i,j] <- abs(mean[i] - mean[j]) / sqrt(var.e * (1 / n[i] + 1 / n[j]))
          t[j,i] <- t[i,j] * sqrt(2)
        }
        j <- j + 1 
     }
     j <- 1
     i <- i + 1   
  }
  k <- nrow(dat)
  i <- 1
  p <- p. <- combination <- P. <- K <- NULL
  while(k != 1){
     p.<- append(p.,k)
     p <- comb(nrow(dat),k)
     while(i <= nrow(comb(nrow(dat),k))){
        combination <- c(p[i,])
        pp <- length(combination)
        while(pp < nrow(dat)){
           combination <- append(combination,0)
           pp <- pp + 1
           }
        P.<- rbind(P.,combination)
        i <- i + 1
        K <- append(K,k)
        }  
     i <- 1
     k <- k - 1
  }
  c <- NULL
  c <- c(rep(alpha,each=nrow(dat)-1))
  k <- nrow(dat)
  l <- i <- 1
  min.p <- pp <- p.value <- w <- v <- p <- KK <- NULL
  while(k != 1){
    while(l <= nrow(comb(nrow(dat),k))){
      pp <- ptukey(t, k , phi.e, lower.tail = FALSE)[comb(nrow(dat),k)[l,],comb(nrow(dat),k)[l,]]
      min.p <- append(min.p,min(pp))
      while(i <= k){
        p.value <- c(pp[i,])
        v <- length(p.value)
        while(v < nrow(dat)){
          p.value <- append(p.value,0)
          v <- v + 1
        }
        w <- rbind(w,p.value)
        i <- i + 1
        KK <- append(KK,k)
      }
      l <- l + 1
      i <- 1
    }
    p <- append(p,k)
    k <- k-1
    l <- 1
  }
  i <- 1
  k <- nrow(dat)
  Q.p <- NULL
  while(k != 1){
    while(l <= nrow(comb(nrow(dat),k))){
      Q <- max(t1[comb(nrow(dat),k)[l,],comb(nrow(dat),k)[l,]])
      Q.p <- append(Q.p,Q)
      l <- l + 1
    }
    k <- k - 1
    l <- 1
  }
  k <- nrow(dat)
  h <- i <- j <- 1
  d <- 0
  q <- matrix(0,ncol = nrow(dat),nrow = nrow(w))
  while(1 != k){
    while(i <= length(comb(nrow(dat),k)) + sum(d)) {
      while(j <= k) {
        if(w[i,j] < c[h]){
          q[i,j] = 1
        }
        j <- j + 1
      }
      j <- 1
      i <- i + 1
    }
    d <- append(d,length(comb(nrow(dat),k)))
    i <- sum(d) + 1       
    k <- k - 1
    h <- h + 1
  }
  k <- j <- m <- nrow(dat)
  i <- l <- n <- 1
  d <- 0
  nn <- nnn <- r <- NULL
  re <- matrix(0,ncol = 1,nrow = length(Q.p))
  while(k != 1){
    while(j <= length(comb(nrow(dat),k)) + sum(d)){
      r <- q[i:j,]   
      j <- j + k
    }
    i <- length(comb(nrow(dat),k)) + sum(d) + 1
    while(m <= nrow(r)){
      if(sum(r[l:m,]) != 0){
        re[n,] <- 2
        nnn <- append(nnn,n)
        n <- n + 1
      }
      if(sum(r[l:m,]) == 0){
        re[n,] <- 1      
        nn <- append(nn,n)
        n <- n + 1
      }
      l <- l + k 
      m <- m + k
    }
    d <- append(d,length(comb(nrow(dat),k)))
    k <- k - 1
    j <- j + (k - 1)
    l <- 1
    m <- k
  }
  k <- nrow(dat)
  i <- 1
  N. <- K. <- NULL
  while(i <= length(nn)){
    N <- nn[i]
    while(N > nrow(comb(nrow(dat),k))){
      N <- {N - nrow(comb(nrow(dat),k))}
      k <- k - 1
    }
    N. <- append(N.,N)
    K. <- append(K.,k)
    i <- i + 1
    k <- nrow(dat)
  }
  i <- j <- 1
  resu <- NULL
  while(i <= length(nn)){
    res <- length(comb(nrow(dat),K.[i])[N.[i],])
    while(res > 2){
      z <- comb(res,res-1)
      while(j <= res){
        resul <- comb(nrow(dat),K.[i])[N.[i],z[j,]]
        v <- length(resul)
        while(v < nrow(dat)){
          resul <- append(resul,0)
          v <- v + 1
        }
        resu <- rbind(resu,resul)
        j <- j + 1  
      }                   
      res <- res - 1
    }
    j<-1
    i <- i + 1
  }
  i <- j <- 1
  if(length(resu)!=0){
    while(i <= nrow(resu)){
      while(j <= nrow(P.)){
        if(sum(abs(P.[j,] - resu[i,])) == 0){
          re[j,] <- 0
        }
        j <- j + 1
      }
      j <- 1
      i <- i + 1
    }
  }
  SNK_re<-re
  K <- matrix(K,ncol = 1)
  colnames(K) <- paste("groups")
  P <- cbind(P.,K)
  p. <- matrix(p.,nrow = length(p.))
  colnames(p.) <- paste("groups")
  c <- matrix(c,nrow = length(c))
  colnames(c) <- paste("significance-level")
  c <- cbind(p.,c)
  KK <- matrix(KK,ncol = 1)
  colnames(KK) <- paste("groups")
  w <- cbind(KK,w)
  min.p <- matrix(min.p,ncol=1)
  colnames(min.p) <- paste("min.p.value")
  min.p <- cbind(P,min.p)
  Q.p <- matrix(Q.p,ncol = 1)
  colnames(Q.p) <- paste("Q-statistic")
  Q.p <- cbind(P,Q.p)
  colnames(re) <- paste("test-result")
  re <- cbind(P,re)
  list(Group = nrow(dat), mean = mean, u.var = u.var, squares = squares, df.e = phi.e, var.e = var.e, 
  test.statistic = t, P = P, p = p., alpha = c, p.value = w, min.p = min.p, Q.p = Q.p, result = re)
#>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>
#  >>>input<<<
#       dat: matrix data with equal variance of each groups
#            (The data with which sample sizes differ is also possible)
#            (NA is put into missing values)
#
#  [External function]
#         comb: The function which searches for combination
#    Factorial: The function which asks for a factorial
#
#  >>>output<<<
#       Group         : the number of groups
#       mean          : the averages for every group
#       u.var         : unbiased variances for every group
#       squares       : sum of squares
#       df.e          : error degrees of freedom
#       var.e         : error variance
#       test.statistic: test statistic
#                       (Upper triangular matrix is test statistic)
#                       (Lower triangular matrix is the standard-ized test statistic)
#       P             : the family set as the guessed object and the number of groups
#       p             : the familiar size to test
#       alpha         : the number of groups and the significance level which 
#                       same regardless of the familiar size
#       p.value       : the number of groups and the significance probability which 
#                       changes for every group
#       min.p         : the minimum significance probability in the combination of 
#                       the group in p.value
#       Q.p           : test statistic which changes with familiar combination
#       result        : test result
#                       (2: significant)
#                       (1: not significant)
#                       (0: not significant by implication)
#
#>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<
#
#                                                         01/17/06  by  Horiuchi
#                                              modified   09/07/06  by  matsu
}

Peritz <- function(dat, alpha = 0.05) 
{
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>                  The method of Peritz                 <<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  i <- j <- 1
  a <- 0
  n <- NULL
  while(i <= nrow(dat)) {
    while(j <= ncol(dat)) {
      if(is.na(dat[i,j]) == FALSE) {
        a <- a + 1
      }
      j <- j + 1
    }
    n <- append(n, a)
    i <- i + 1
    j <- 1
    a <- 0
  }
  i <- 1
  mean <- u.var <- NULL
  while(i <= length(n)){
    mean <- append(mean,mean(t(dat[i,1:n[i]])))
    u.var <- append(u.var,var(t(dat[i,1:n[i]])))
    i <- i + 1
  }
  phi.e <- sum(n) - length(n)
  squares <- sum((n - 1) * u.var)
  var.e <- squares / phi.e
  t <-t1<- matrix(0, ncol = length(n), nrow = length(n))
  colnames(t)<-colnames(t1) <- paste("Group",1:ncol(t))
  rownames(t)<-rownames(t1) <- paste("Group",1:nrow(t))
  i <- j <- 1
  while(i <= length(n)) {
    while(j <= length(n)) {
      if(j > i){
        t[i,j] <- abs(mean[i] - mean[j]) / sqrt(var.e * (1 / n[i] + 1 / n[j]))
        t1[i,j]<-t[j,i] <- t[i,j] * sqrt(2)
      }
      j <- j + 1 
    }
    j <- 1
    i <- i + 1   
  }
  k <- nrow(dat)
  i <- 1
  p <- p. <- combination <- P. <- K <- NULL
  while(k != 1){
    p.<- append(p.,k)
    p <- comb(nrow(dat),k)
    while(i <= nrow(comb(nrow(dat),k))){
      combination <- c(p[i,])
      pp <- length(combination)
        while(pp < nrow(dat)){
          combination <- append(combination,0)
          pp <- pp + 1
        }
        P.<- rbind(P.,combination)
        i <- i + 1
        K <- append(K,k) 
    }  
    i <- 1
    k <- k - 1
  }
  i <- n <- nrow(dat)
  c1 <- c2 <- NULL
  c2 <- c(rep(alpha,each=nrow(dat)-1))
  while(2 <= i){
    if(2 <= i && i <= n - 2){
      alpha <- 1 - {(1 - 0.05)^(i / n)}
    }
    c1 <- append(c1,alpha)
    i <- i - 1
  }
  k <- nrow(dat)
  l <- i <- 1
  min.p <- pp <- p.value <- w <- v <- p <- KK <- NULL
  while(k != 1){
    while(l <= nrow(comb(nrow(dat),k))){
      pp <- ptukey(t, k , phi.e, lower.tail = FALSE)[comb(nrow(dat),k)[l,],comb(nrow(dat),k)[l,]]
      min.p <- append(min.p,min(pp))
      while(i <= k){
        p.value <- c(pp[i,])
        v <- length(p.value)
        while(v < nrow(dat)){
          p.value <- append(p.value,0)
          v <- v + 1
        }
        w <- rbind(w,p.value)
        i <- i + 1
        KK <- append(KK,k)
      }
      l <- l + 1
      i <- 1
    }
    p <- append(p,k)
    k <- k-1
    l <- 1
  }
  i <- 1
  k <- nrow(dat)
  Q.p <- NULL
  while(k != 1){
    while(l <= nrow(comb(nrow(dat),k))){
      Q <- max(t1[comb(nrow(dat),k)[l,],comb(nrow(dat),k)[l,]])
      Q.p <- append(Q.p,Q)
      l <- l + 1
    }
    k <- k - 1
    l <- 1
  }
  Peri <- NULL
  number<-1
  c <-c1
  while(number<=2){
    k <- nrow(dat)
    h <- i <- j <- 1
    d <- 0
    q <- matrix(0,ncol = nrow(dat),nrow = nrow(w))
    while(1 != k){
      while(i <= length(comb(nrow(dat),k)) + sum(d)) {
        while(j <= k) {
          if(w[i,j] < c[h]){
            q[i,j] = 1
          }
          j <- j + 1
        }
        j <- 1
        i <- i + 1
      }
      d <- append(d,length(comb(nrow(dat),k)))
      i <- sum(d) + 1       
      k <- k - 1
      h <- h + 1
    }
    k <- j <- m <- nrow(dat)
    i <- l <- n <- 1
    d <- 0
    nn <- nnn <- r <- NULL
    re <- matrix(0,ncol = 1,nrow = length(Q.p))
    while(k != 1){
      while(j <= length(comb(nrow(dat),k)) + sum(d)){
        r <- q[i:j,]   
        j <- j + k
      }
      i <- length(comb(nrow(dat),k)) + sum(d) + 1
      while(m <= nrow(r)){
        if(sum(r[l:m,]) != 0){
          re[n,] <- 2
           nnn <- append(nnn,n)
           n <- n + 1
        }
        if(sum(r[l:m,]) == 0){
          re[n,] <- 1      
          nn <- append(nn,n)
          n <- n + 1
        }
        l <- l + k 
        m <- m + k
      }
      d <- append(d,length(comb(nrow(dat),k)))
      k <- k - 1
      j <- j + (k - 1)
      l <- 1
      m <- k
    }
    k <- nrow(dat)
    i <- 1
    N. <- K. <- NULL
    while(i <= length(nn)){
      N <- nn[i]
      while(N > nrow(comb(nrow(dat),k))){
        N <- {N - nrow(comb(nrow(dat),k))}
        k <- k - 1
      }
      N. <- append(N.,N)
      K. <- append(K.,k)
      i <- i + 1
      k <- nrow(dat)
    }
    i <- j <- 1
    resu <- NULL
    while(i <= length(nn)){
      res <- length(comb(nrow(dat),K.[i])[N.[i],])
      while(2 < res){
        z <- comb(res,res - 1)
        while(j <= res){
          resul <- comb(nrow(dat),K.[i])[N.[i],z[j,]]
          v <- length(resul)
          while(v < nrow(dat)){
            resul <- append(resul,0)
            v <- v + 1
          }
          resu <- rbind(resu,resul)
          j <- j + 1  
        }                   
        res <- res - 1
      }
      j<-1
      i <- i + 1
    }
    i <- j <- 1
    if(length(resu)!=0){
      while(i <= nrow(resu)){
        while(j <= nrow(P.)){
          if(sum(abs(P.[j,] - resu[i,])) == 0){
            re[j,] <- 0
          }
          j <- j + 1
        }
        j <- 1
        i <- i + 1
      }
    }
    Peri<-cbind(Peri,re)
    c<-c2
    number<-number+1
  }
  TW_re<-Peri[,1]
  SNK_re<-Peri[,2]
  tw<-TW_re
  s<-SNK_re
  p<-n1<-NULL
  i<-1
  while(i<=nrow(P.)){
    if(tw[i]==s[i] | (tw[i]==0&&s[i]==1)){
      p[i]<-s[i]
    }
    else{
    p[i]<-5
    n1<-append(n1,i)
    }
    i<-i+1
  }
  i<-1
  n2<-NULL
  while(i<=length(n1)){
    if(min.p[n1[i]] <  c1[nrow(dat)+1-(K[n1[i]])]){
      p[n1[i]]<-2
    }
    else{
    n2<-append(n2,n1[i])
    }
    i<-i+1
  }
  if(length(n2)!=0){
    H<-1
    A1<-NULL
    a<-A1<-c(1:nrow(dat))[-P.[n2[H],][1:K[n2[H]]]]
    while(H<=length(n2)){
      if(length(a)==2){
        m <- n <- length(A1)
        while(n < nrow(dat)){
          A1 <- append(A1,0)
          n <- n + 1
        }     
        i<-1
        nA1<-NULL
        while(i<=nrow(P.)){
          if(sum(abs(P.[i,]-A1))==0){
            nA1<-i
          }
          i<-i+1
        }
        if(min.p[nA1] <  c1[nrow(dat)+1-m]){
          p[n2[H]]<-2
        }
        else{
          p[n2[H]]<-1
        }
      }
      if(2<length(a)){
        m <- n <- length(A1)
        while(n < nrow(dat)){
          A1 <- append(A1,0)
          n <- n + 1
        }     
        i<-1
        nA1<-NULL
        while(i<=nrow(P.)){
          if(sum(abs(P.[i,]-A1))==0){
            nA1<-append(nA1,i)
          }
          i<-i+1
        }
        nn<-nA1
        k <- nrow(dat)
        i <- 1
        N. <- K. <- NULL
        while(i <= length(nn)){
          N <- nn[i]
          while(N >= nrow(comb(nrow(dat),k))){
            N <- {N - nrow(comb(nrow(dat),k))}
            k <- k - 1
          }
          N. <- append(N.,N)
          K. <- append(K.,k)
          i <- i + 1
          k <- nrow(dat)
        }
        i <- j <- 1
        resu<-NULL
        while(i <= length(nn)){
          res <- length(comb(nrow(dat),K.[i])[N.[i],])
          while(res > 2){
            z <- comb(res,res - 1)
            while(j <= res){
              resul <- comb(nrow(dat),K.[i])[N.[i],z[j,]]
              v <- length(resul)
              m <- append(m,v)
              while(v < nrow(dat)){
                resul <- append(resul,0)
                v <- v + 1
              }
              resu <- rbind(resu,resul)
              j <- j + 1  
            }                   
            res <- res - 1
          }
          j<-1
          i <- i + 1
        }
        A1<-rbind(A1,resu)
        i<-1
        j<-1
        nA1<-NULL
        while(j<=nrow(A1)){
          while(i<=nrow(P.)){
            if(sum(abs(P.[i,]-A1[j,]))==0){
              nA1<-append(nA1,i)
            }
            i<-i+1
          }
          i<-1
          j<-j+1
        }
        j<-1
        C<-NULL
        while(j<=nrow(A1)){
          if(min.p[nA1[j]] < c1[nrow(dat)+1-m[j]]){
            C<-append(C,2)
          }
          j<-j+1
        }
        if(sum(C)==2*nrow(A1)){
          p[n2[H]]<-2
        }
        else{
          p[n2[H]]<-1
        }
      }
      H<-H+1
    }
  }
  K <- matrix(K,ncol = 1)
  colnames(K) <- paste("groups")
  P <- cbind(P.,K)
  p. <- matrix(p.,nrow = length(p.))
  colnames(p.) <- paste("groups")
  c1 <- matrix(c1,nrow = length(c))
  c2 <- matrix(c2,nrow = length(c))
  colnames(c1) <- paste("TW_significance-level")
  colnames(c2) <- paste("SNK_significance-level")
  c<-cbind(c1,c2)
  c <- cbind(p.,c)
  KK <- matrix(KK,ncol = 1)
  colnames(KK) <- paste("groups")
  w <- cbind(KK,w)
  min.p <- matrix(min.p,ncol=1)
  colnames(min.p) <- paste("min.p.value")
  min.p <- cbind(P,min.p)
  Q.p <- matrix(Q.p,ncol = 1)
  colnames(Q.p) <- paste("Q-statistic")
  Q.p <- cbind(P,Q.p)
  TW_re<-Peri[,1]
  SNK_re<-Peri[,2]
  TW_re <- matrix(TW_re,nrow = nrow(P))
  SNK_re <- matrix(SNK_re,nrow = nrow(P))
  Peri_re <- matrix(p,nrow = nrow(P))
  colnames(TW_re) <- paste("TW_result")
  colnames(SNK_re) <- paste("SNK_result")
  colnames(Peri_re) <- paste("Peritz_result")
  re <- cbind(P,TW_re,SNK_re,Peri_re)
  list(Group = nrow(dat), mean = mean, u.var = u.var, squares = squares, df.e = phi.e, var.e = var.e, 
  test.statistic = t, P = P, p = p., alpha = c, p.value = w, min.p = min.p, Q.p = Q.p, result = re)
#>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>
#  >>>input<<<
#       dat: matrix data with equal variance of each groups
#            (The data with which sample sizes differ is also possible)
#            (NA is put into missing values)
#
#  [External function]
#         comb: The function which searches for combination
#    Factorial: The function which asks for a factorial
#
#  >>>output<<<
#       Group         : the number of groups
#       mean          : the averages for every group
#       u.var         : unbiased variances for every group
#       squares       : sum of squares
#       df.e          : error degrees of freedom
#       var.e         : error variance
#       test.statistic: test statistic
#                       (Upper triangular matrix is test statistic)
#                       (Lower triangular matrix is the standard-ized test statistic)
#       P             : the family set as the guessed object and the number of groups
#       p             : the familiar size to test
#       alpha         : the number of groups and the significance level which 
#                       changed according to the familiar size(Tukey-Welsch) and
#                       same regardless of the familiar size(SNK) 
#       p.value       : the number of groups and the significance probability which
#                       changes for every group
#       min.p         : the minimum significance probability in the combination of 
#                       the group in p.value
#       Q.p           : test statistic which changes with familiar combination
#       result        : test result(Peritz result and Tukey-Welsch result and SNK result)                       
#                       (2: significant)
#                       (1: not significant)
#                       (0: not significant by implication)
#
#>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<
#
#                                                         01/17/06  by  Horiuchi
#                                              modified   09/07/06  by  matsu
}

