Simon <- function(p0,p1,Alpha,Beta,N) {

#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>>                                                                       <<<<<
#>>>>>    Optimal two-stage designs for phase II clinical trials by Simon    <<<<<
#>>>>>                                                                       <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

 Opt_EN <- 10000
 Opt_r1 <- -10000
 Opt_r <- -10000
 Opt_PET <- 10000
 Opt_Alpha <- 10000
 Opt_Beta <- 10000
 Opt_n <- -10000
 Opt_n1 <- -10000

 Min_n1 <- 10000
 Min_n <- 10000
 Min_r1 <- -10000
 Min_r <- -10000
 Min_EN <- 10000
 Min_PET <- 10000
 Min_Alpha <- 10000
 Min_Beta <- 10000
 Max_EN <- -10000

 tEN <- 10000
 pl_n <- c()
 pl_EN0 <- c()

 s <- 1
 t <- s + 1
 while(t <= N) {
  s <- 1
  while(s <= t-1) {
   n2 <- t - s
   for(i in 0:s) {
    for(j in i+1:t) {
     a01 <- pbinom(i,s,p0)
     a11 <- pbinom(i,s,p1)
     a02 <- 0
     for(k in i+1:min(s,j)) {
      a02 <- a02 + (dbinom(k,s,p0) * pbinom(j-k,n2,p0))
     }
     a12 <- 0
     for(k in i+1:min(s,j)) {
      a12 <- a12 + (dbinom(k,s,p1) * pbinom(j-k,n2,p1))
     }
     Ap0 <- 1 - ( a01 + a02 )
     Ap1 <- 1 - ( a11 + a12 ) 
     
     if((Ap0 <= Alpha) && (Ap1 >= 1 - Beta)) {
      EN0 <- s + ( 1 - a01 ) * n2 
      if(EN0 < tEN) {
       tEN <- EN0
      }
      if(EN0 < Opt_EN) {
       Opt_EN <- EN0
       Opt_r1 <- i
       Opt_r <- j
       Opt_PET <- a01
       Opt_Alpha <- Ap0
       Opt_Beta <- Ap1
       Opt_n <- t
       Opt_n1 <- s
      }
 
      if(Min_n >= t) {
       Min_n <- t 
       {
       if(EN0 < Min_EN) {
        Min_EN <- EN0
        Min_n1 <- s
        Min_r1 <- i
        Min_r <- j
        Min_EN <- EN0
        Min_PET <- a01
        Min_Alpha <- Ap0
        Min_Beta <- Ap1
       }
       }
      }

     }
    }
   }
   s <- s + 1
  }

  if(tEN != 10000) {
   pl_EN0[t] <- tEN
   pl_n[t] <- t
   {
   if(pl_EN0[t] > Max_EN) {
    Max_EN <- pl_EN0[t]
   }
   }
  }

  tEN <- 10000
  t <- t + 1
 }

 plot(pl_n, pl_EN0, type="o",xaxs = "i", yaxs = "i",xlim = c(Min_n-1, N+1), ylim = c(Opt_EN-1, Max_EN+1),xlab = "Sample Size", ylab = "EN(p0)", main = "Optimal/Minitmax Two-Stage Phase II Designs") 
 par(new=T)
 text(Min_n,Min_EN,labels="Minitmax",col="blue")
 text(Opt_n,Opt_EN,labels="Optimal",col="red")
 ans <- list(Ootimal_r1 = Opt_r1, Optimal_n1 = Opt_n1, Optimal_r = Opt_r, Optimal_n = Opt_n, Optimal_EN = Opt_EN, Optimal_PET = Opt_PET, Optimal_Alpha = Opt_Alpha, Optimal_power = Opt_Beta, Minimax_r1 = Min_r1, Minimax_n1 = Min_n1, Minimax_r = Min_r, Minimax_n = Min_n, Minimax_EN = Min_EN, Minimax_PET = Min_PET, Minimax_Alpha = Min_Alpha, Minimax_power = Min_Beta)
 return(ans)

#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#   >>> input <<<
#            p0: disappointment success rate
#            p1: promising success rate
#         Alpha: blunder probability of first class
#          Beta: blunder probability of second class
#             N: the upper limit of the number of the greatest cases
#
#   >>> output <<<
#    Optimal_r1: the number of the first grade border examples of optimal designs
#    Optimal_n1: the number of the first grade cases of optimal designs
#     Optimal_r: the number of the second grade border examples of optimal designs
#     Optimal_n: the number of the greatest cases of optimal designs
#    Optimal_EN: expectation of the total number of people to examine of optimal designs
#   Optimal_PET: a cumulative distribution function of binominal distribution of optimal designs
# Optimal_Alpha: blunder probability of first class of optimal designs
# Optimal_power: the destection power of optimal designs
#
#    Minimax_r1: the number of the first grade border examples of minimax designs
#    Minimax_n1: the number of the first grade cases of minimax designs
#     Minimax_r: the number of the second grade border examples of minimax designs
#     Minimax_n: the number of the greatest cases of minimax designs
#    Minimax_EN: expectation of the total number of people to examine of minimax designs
#   Minimax_PET: a cumulative distribution function of binominal distribution of minimax designs
# Minimax_Alpha: blunder probability of first class of minimax designs
# Minimax_power: the destection power of minimax designs
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                                                 03/11/07   by Nomura

}
