tukey <- function(dat, alpha = 0.05) 
{
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>          The method of Tukey(Tukey-Kramer)            <<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   i <- j <- 1
   a <- 0
   n <- NULL
   while(i <= nrow(dat)) {
      while(j <= ncol(dat)) {
         if(is.na(dat[i,j]) == FALSE) {
            a <- a + 1
         }
         j <- j + 1
      }
      n <- append(n, a)
      i <- i + 1
      j <- 1
      a <- 0
   }
   i <- 1
   m <- v <- NULL
   while(i <= length(n)){
      m <- append(m,mean(t(dat[i,1:n[i]])))
      v <- append(v,var(t(dat[i,1:n[i]])))
      i <- i + 1
   }
   phi.e <- sum(n) - length(n)
   squares <- sum((n - 1) * v)
   var.e <- squares / phi.e
   t <- p <- q <- matrix(0, ncol = length(n), nrow = length(n))
   colnames(t) <- colnames(p) <- colnames(q)<- paste("Group",1:ncol(t))
   rownames(t) <- rownames(p) <- rownames(q)<- paste("Group",1:nrow(t))
   i <- j <- 1
   while(i <= length(n)) {
      while(j <= length(n)) {
         if(j > i){
            t[i,j] <- abs(m[i] - m[j]) / sqrt(var.e * (1 / n[i] + 1 / n[j]))
            t[j,i] <- t[i,j] * sqrt(2)
            p[i,j] <- ptukey(t[j,i], length(n), phi.e, lower.tail = FALSE)
               if(p[i,j] < alpha){
                  q[i,j] = 1
               }
         }
         j <- j + 1 
      }
      j <- 1
      i <- i + 1   
   }
   list(alpha = alpha , Group = length(n), mean = m, u.var = v, squares = squares, 
        phi.e = phi.e, var.e = var.e, test.statistic = t, p.value = p, resurt = q)
#>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>
#  >>>input<<<
#       dat: matrix data with equal variance of each groups
#            (The data with which sample sizes differ is also possible)
#            (NA is put into missing values)
#
#  >>>output<<<
#       alpha         : significance level (changeable)  
#       Group         : the number of groups
#       mean          : the averages for every group
#       u.var         : unbiased variances for every group
#       squares       : sum of squares
#       phi.e         : error degrees of freedom
#       var.e         : error variance
#       test.statistic: test statistic
#                       (Upper triangular matrix is calculated test statistic)
#                       (Lower triangular matrix is standard-ized test statistic)
#       p.value       : significance probability of standard-ized test statistic
#       result        : test result
#                       (1: significant)
#                       (0: not significant)
#
#>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<
#
#                                                         02/19/04  by  Horiuchi
#                                              modified   09/07/06  by  matsu
}

tukey.w <- function(dat, alpha = 0.05) 
{
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>              The method of Tukey-Welsch               <<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  i <- j <- 1
  a <- 0
  n <- NULL
  while(i <= nrow(dat)) {
     while(j <= ncol(dat)) {
        if(is.na(dat[i,j]) == FALSE) {
           a <- a + 1
        }
        j <- j + 1
     }
     n <- append(n, a)
     i <- i + 1
     j <- 1
     a <- 0
  }
  i <- 1
  mean <- u.var <- NULL
  while(i <= length(n)){
     mean <- append(mean,mean(t(dat[i,1:n[i]])))
     u.var <- append(u.var,var(t(dat[i,1:n[i]])))
     i <- i + 1
  }
  phi.e <- sum(n) - length(n)
  squares <- sum((n - 1) * u.var)
  var.e <- squares / phi.e
  t <-t1<- matrix(0, ncol = length(n), nrow = length(n))
  colnames(t)<-colnames(t1) <- paste("Group",1:ncol(t))
  rownames(t)<-rownames(t1) <- paste("Group",1:nrow(t))
  i <- j <- 1
  while(i <= length(n)) {
     while(j <= length(n)) {
        if(j > i){
           t1[i,j]<-t[i,j] <- abs(mean[i] - mean[j]) / sqrt(var.e * (1 / n[i] + 1 / n[j]))
           t[j,i] <- t[i,j] * sqrt(2)
        }
        j <- j + 1 
     }
     j <- 1
     i <- i + 1   
  }
  k <- nrow(dat)
  i <- 1
  p <- p. <- combination <- P. <- K <- NULL
  while(k != 1){
     p.<- append(p.,k)
     p <- comb(nrow(dat),k)
     while(i <= nrow(comb(nrow(dat),k))){
        combination <- c(p[i,])
        pp <- length(combination)
        while(pp < nrow(dat)){
           combination <- append(combination,0)
           pp <- pp + 1
           }
        P.<- rbind(P.,combination)
        i <- i + 1
        K <- append(K,k)
        }  
     i <- 1
     k <- k - 1
  }
  i <- n <- nrow(dat)
  c <- NULL
  while(2 <= i){
     if(2 <= i && i <= n - 2){
        alpha2 <- 1 - {(1 - alpha)^(i / n)}
     } else alpha2 <- alpha
     c <- append(c,alpha2)
     i <- i - 1
  }
  k <- nrow(dat)
  l <- i <- 1
  min.p <- pp <- p.value <- w <- v <- p <- KK <- NULL
  while(k != 1){
     while(l <= nrow(comb(nrow(dat),k))){
        pp <- ptukey(t, k , phi.e, lower.tail = FALSE)[comb(nrow(dat),k)[l,],comb(nrow(dat),k)[l,]]
        min.p <- append(min.p,min(pp))
        while(i <= k){
           p.value <- c(pp[i,])
           v <- length(p.value)
           while(v < nrow(dat)){
              p.value <- append(p.value,0)
              v <- v + 1
           }
           w <- rbind(w,p.value)
           i <- i + 1
           KK <- append(KK,k)
        }
        l <- l + 1
        i <- 1
     }
     p <- append(p,k)
     k <- k-1
     l <- 1
  }
  i <- 1
  k <- nrow(dat)
  Q.p <- NULL
  while(k != 1){
     while(l <= nrow(comb(nrow(dat),k))){
        Q <- max(t1[comb(nrow(dat),k)[l,],comb(nrow(dat),k)[l,]])
        Q.p <- append(Q.p,Q)
        l <- l + 1
     }
  k <- k - 1
  l <- 1
  }
  k <- nrow(dat)
  h <- i <- j <- 1
  d <- 0
  q <- matrix(0,ncol = nrow(dat),nrow = nrow(w))
  while(1 != k){
     while(i <= length(comb(nrow(dat),k)) + sum(d)) {
        while(j <= k) {
           if(w[i,j] < c[h]){
              q[i,j] = 1
           }
           j <- j + 1
        }
        j <- 1
        i <- i + 1
     }
     d <- append(d,length(comb(nrow(dat),k)))
     i <- sum(d) + 1       
     k <- k - 1
     h <- h + 1
  }
  k <- j <- m <- nrow(dat)
  i <- l <- n <- 1
  d <- 0
  nn <- nnn <- r <- NULL
  re <- matrix(0,ncol = 1,nrow = length(Q.p))
  while(k != 1){
     while(j <= length(comb(nrow(dat),k)) + sum(d)){
        r <- q[i:j,]   
        j <- j + k
     }
     i <- length(comb(nrow(dat),k)) + sum(d) + 1
     while(m <= nrow(r)){
        if(sum(r[l:m,]) != 0){
           re[n,] <- 2
           nnn <- append(nnn,n)
           n <- n + 1
        }
        if(sum(r[l:m,]) == 0){
           re[n,] <- 1      
           nn <- append(nn,n)
           n <- n + 1
        }
        l <- l + k 
        m <- m + k
     }
     d <- append(d,length(comb(nrow(dat),k)))
     k <- k - 1
     j <- j + (k - 1)
     l <- 1
     m <- k
  }
  k <- nrow(dat)
  i <- 1
  N. <- K. <- NULL
  while(i <= length(nn)){
     N <- nn[i]
     while(N > nrow(comb(nrow(dat),k))){
        N <- {N - nrow(comb(nrow(dat),k))}
        k <- k - 1
     }
     N. <- append(N.,N)
     K. <- append(K.,k)
     i <- i + 1
     k <- nrow(dat)
  }
  i <- j <- 1
  resu <- NULL
  while(i <= length(nn)){
    res <- length(comb(nrow(dat),K.[i])[N.[i],])
    while(res > 2){
      z <- comb(res,res - 1)
      while(j <= res){
        resul <- comb(nrow(dat),K.[i])[N.[i],z[j,]]
        v <- length(resul)
        while(v < nrow(dat)){
          resul <- append(resul,0)
          v <- v + 1
        }
        resu <- rbind(resu,resul)
        j <- j + 1  
      }                   
      res <- res - 1
    }
    j<-1
    i <- i + 1
  }
  i <- j <- 1
  if(length(resu)!=0){
    while(i <= nrow(resu)){
      while(j <= nrow(P.)){
        if(sum(abs(P.[j,] - resu[i,])) == 0){
          re[j,] <- 0
        }
        j <- j + 1
      }
      j <- 1
      i <- i + 1
    }
  }
  TW_re<-re
  K <- matrix(K,ncol = 1)
  colnames(K) <- paste("groups")
  P <- cbind(P.,K)
  p. <- matrix(p.,nrow = length(p.))
  colnames(p.) <- paste("groups")
  c <- matrix(c,nrow = length(c))
  colnames(c) <- paste("significance-level")
  c <- cbind(p.,c)
  KK <- matrix(KK,ncol = 1)
  colnames(KK) <- paste("groups")
  w <- cbind(KK,w)
  min.p <- matrix(min.p,ncol=1)
  colnames(min.p) <- paste("min.p.value")
  min.p <- cbind(P,min.p)
  Q.p <- matrix(Q.p,ncol = 1)
  colnames(Q.p) <- paste("Q-statistic")
  Q.p <- cbind(P,Q.p)
  colnames(re) <- paste("test-result")
  re <- cbind(P,re)
  list(Group = nrow(dat), mean = mean, u.var = u.var, squares = squares, df.e = phi.e, var.e = var.e, 
  test.statistic = t, P = P, p = p., alpha = c, p.value = w, min.p = min.p, Q.p = Q.p, result = re)
#>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>
#  >>>input<<<
#       dat: matrix data with equal variance of each groups
#            (The data with which sample sizes differ is also possible)
#            (NA is put into missing values)
#
#  [External function]
#         comb: The function which searches for combination
#    Factorial: The function which asks for a factorial
#
#  >>>output<<<
#       Group         : the number of groups
#       mean          : the averages for every group
#       u.var         : unbiased variances for every group
#       squares       : sum of squares
#       df.e          : error degrees of freedom
#       var.e         : error variance
#       test.statistic: test statistic
#                       (Upper triangular matrix is test statistic)
#                       (Lower triangular matrix is the standard-ized test statistic)
#       P             : the family set as the guessed object and the number of groups
#       p             : the familiar size to test
#       alpha         : the number of groups and the significance level which changed according to the familiar size
#       p.value       : the number of groups and the significance probability which changes for every group
#       min.p         : the minimum significance probability in the combination of the group in p.value
#       Q.p           : test statistic which changes with familiar combination
#       result        : test result
#                       (2: significant)
#                       (1: not significant)
#                       (0: not significant by implication)
#
#>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<
#
#                                                         02/19/04  by  Horiuchi
#                                                         01/27/06  by  Horiuchi
#                                              modified   09/07/06  by  matsu
}

comb <- function(a, b)
{
#It is used by the method of Tukey-Welsch.
#                         02/19/04  by  Horiuchi
   x <- matrix(0, ncol = b, nrow = Factorial(a)/(Factorial(a - b) * Factorial(b)))
   x[1,] <- c(1:b)
   i <- l <- b
   j <- 1
   while(x[j,i] <= a && j < nrow(x)){
      x[j + 1,] <- x[j,]
      if(x[j,i] == a){
         l <- i - 1
         while(x[j,l + 1] - x[j,l] == 1) {
            x[j + 1,l - 1] <- x[j,l - 1] + 1
            l <- l - 1
         }   
         x[j + 1,l] <- x[j,l] + 1
         l <- l + 1
         while(l <= b){
            x[j + 1,l] <- x[j + 1,l - 1] + 1
            if(x[j + 1,l] > a){
               x[j + 1,] <- x[1,] + x[j,1] 
            }
            l <- l + 1         
         }
      }
      else{
         x[j + 1,i] <- x[j + 1,i] + 1
      }
      j <- j + 1
   }
   x
} 

Factorial <- function(n)
{
#It is used with a combination function.
#                  02/19/04  by  Horiuchi
   f <- 1 
   if(n!=0){ 
      while(n!=1){
         f <- f * n
         n <- n - 1
      }
   } 
   f
}
