zoukahou <- function(x, y, method = "adjr2")
{
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                                       <<<<<
#>>>>   Stepwise linear regression procedure (zouka-hou)    <<<<<
#>>>>                                                       <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	imeth <- (match(method, c("adjr2", "adj2r2", "AIC", "FPE"), 0))
	if(imeth == 0)
		stop("Invalid method")
	kijun <- function(x, y, imeth)
	{
		if(imeth == 1) {
			kijun <- adjr2(x, y)
		}
		else if(imeth == 2) {
			kijun <- adj2r2(x, y)
		}
		else if(imeth == 3) {
			kijun <- -1 * aic(x, y)
		}
		else if(imeth == 4) {
			kijun <- -1 * FPE(x, y)
		}
		kijun
	}
	if(is.vector(x) == T) {
		Max <- kijun(x, y, imeth)
		MaxX <- 1
	}
	else {
		p <- ncol(x)
		bangou <- c(1:p)
		Max <- kijun(x[, 1], y, imeth)
		Maxi <- 1
		for(i in c(2:p)) {
			k <- kijun(x[, i], y, imeth)
			if(k > Max) {
				Max <- k
				Maxi <- i
			}
		}
		bangou <- bangou[ - Maxi]
		Maxbind <- x[, Maxi]
		z <- ncol(x) - 1
		x <- x[,  - Maxi]
		MaxX <- Maxi
		MaxX2 <- Maxi
		while(z > 1) {
			Max2 <- -1e+99
			i <- 1
			while(i <= z) {
				xw <- cbind(Maxbind, x[, i])
				k <- kijun(xw, y, imeth)
				if(k >= Max2) {
				  Max2 <- k
				  MaxX2W <- c(MaxX2, bangou[i])
				  Maxbindw <- xw
				  Maxi2 <- i
				}
				i <- i + 1
			}
			MaxX2 <- MaxX2W
			bangou <- bangou[ - Maxi2]
			Maxbind <- Maxbindw
			z <- ncol(x) - 1
			x <- x[,  - Maxi2]
			if(Max2 > Max) {
				Max <- Max2
				MaxX <- MaxX2
			}
		}
		xw <- cbind(Maxbind, x)
		k <- kijun(xw, y, imeth)
		if(k >= Max) {
			Max <- k
			MaxX <- c(MaxX2, bangou)
		}
	}
	teisuu <- kijun(NULL, y, imeth)
	if(teisuu > Max) {
		Max <- teisuu
		MaxX <- NA
	}
	if(imeth == 1) {
		ans <- list(adjr2 = Max, kumiawase = MaxX)
	}
	else if(imeth == 2) {
		ans <- list(adj2r2 = Max, kumiawase = MaxX)
	}
	else if(imeth == 3) {
		Max <- -1 * Max
		ans <- list(AIC = Max, kumiawase = MaxX)
	}
	else if(imeth == 4) {
		Max <- -1 * Max
		ans <- list(FPE = Max, kumiawase = MaxX)
	}
	ans	#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#           x: independent variables matrix
#           y: dependent variable
#      method: A criterion among the following list;
#              adjr2: multiple correlation coefficient adjusted for DF
#             adj2r2: multiple correlation coefficient double adjusted for DF
#                AIC: Akaike's information criterion
#                FPE: final prediction error
#
#  >>> output <<<
# [criterion]: the value of [criterion] on the optimal result
#   kumiawase: numbers of variables on the optimal result
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                       03/20/00  by  Ohmura, Yamahara
#                    modified for R     11/08/02  by  matsu
}

gensyouhou <- function(x, y, method = "adjr2")
{
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                                       <<<<<
#>>>>   Stepwise linear regression procedure (gensyou-hou)  <<<<<
#>>>>                                                       <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	imeth <- (match(method, c("adjr2", "adj2r2", "AIC", "FPE"), 0))
	if(imeth == 0)
		stop("Invalid method")
	kijun <- function(x, y, imeth)
	{
		if(imeth == 1) {
			kijun <- adjr2(x, y)
		}
		else if(imeth == 2) {
			kijun <- adj2r2(x, y)
		}
		else if(imeth == 3) {
			kijun <- -1 * aic(x, y)
		}
		else if(imeth == 4) {
			kijun <- -1 * FPE(x, y)
		}
		kijun
	}
	if(is.vector(x) == T) {
		MaxMax <- kijun(x, y, imeth)
		Maxi <- 1
	}
	else {
		p <- ncol(x)
		bangou <- c(1:p)
		MaxMax <- kijun(x, y, imeth)
		Maxi <- bangou
		Max <- -1e+99
		for(i in c(1:p)) {
			k <- kijun(x[,  - i], y, imeth)
			if(k > Max) {
				Max <- k
				hazurei <- i
				MaxB <- bangou[ - i]
			}
		}
		z <- ncol(x) - 1
		bangou <- MaxB
		x <- x[,  - hazurei]
		if(Max > MaxMax) {
			MaxMax <- Max
			Maxi <- MaxB
		}
		while(z > 1) {
			i <- 0
			Max <- -1e+99
			while(i < z) {
				i <- i + 1
				B <- x[,  - i]
				k <- kijun(B, y, imeth)
				if(k > Max) {
				  Max <- k
				  MaxB <- bangou[ - i]
				  hazurei <- i
				}
			}
			z <- ncol(x) - 1
			x <- x[,  - hazurei]
			bangou <- MaxB
			if(Max > MaxMax) {
				MaxMax <- Max
				Maxi <- MaxB
			}
		}
	}
	n <- NULL
	teisuu <- kijun(n, y, imeth)
	if(teisuu > MaxMax) {
		MaxMax <- teisuu
		Maxi <- NA
	}
	if(imeth == 1) {
		ans <- list(adjr2 = MaxMax, kumiawase = Maxi)
	}
	else if(imeth == 2) {
		ans <- list(adj2r2 = MaxMax, kumiawase = Maxi)
	}
	else if(imeth == 3) {
		MaxMax <- -1 * MaxMax
		ans <- list(AIC = MaxMax, kumiawase = Maxi)
	}
	else if(imeth == 4) {
		MaxMax <- -1 * MaxMax
		ans <- list(FPE = MaxMax, kumiawase = Maxi)
	}
	ans	#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#           x: independent variables matrix
#           y: dependent variable
#      method: A criterion among the following list;
#              adjr2: multiple correlation coefficient adjusted for DF
#             adj2r2: multiple correlation coefficient double adjusted for DF
#                AIC: Akaike's information criterion
#                FPE: final prediction error
#
#  >>> output <<<
# [criterion]: the value of [criterion] on the optimal result
#   kumiawase: numbers of variables on the optimal result
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                       03/20/00  by  Ohmura, Yamahara
#                    modified for R     11/08/02  by  matsu
}

adjr2 <- function(x, y)
{
# Using from genshouhou, zoukahou, genzouhou, zougenhou
#                                03/20/00 by Ohmura, Yamahara
	if(is.null(x) == T) {
		n <- length(y)
		p <- 0
		q <- rep(1, n)
		r <- lsfit(q, y, intercept = F)
		s <- r$r^2
		se <- sum(s)
		a <- (y - mean(y))^2
		st <- sum(a)
		z <- (1 - (se/(n - p - 1))/(st/(n - 1)))
		z
	}
	else {
		if(is.vector(x) == T) {
			n <- length(x)
			p <- 1
			r <- lsfit(x, y)
			s <- r$r^2
			se <- sum(s)
			a <- (y - mean(y))^2
			st <- sum(a)
			z <- (1 - (se/(n - p - 1))/(st/(n - 1)))
			z
		}
		else {
			n <- nrow(x)
			p <- ncol(x)
			r <- lsfit(x, y)
			s <- r$r^2
			se <- sum(s)
			a <- (y - mean(y))^2
			st <- sum(a)
			z <- (1 - (se/(n - p - 1))/(st/(n - 1)))
			z
		}
	}
}

adj2r2 <- function(x, y)
{
# Using from genshouhou, zoukahou, genzouhou, zougenhou
#                                03/20/00 by Ohmura, Yamahara
	if(is.null(x) == T) {
		n <- length(y)
		p <- 0
		q <- rep(1, n)
		r <- lsfit(q, y, intercept = F)
		s <- r$r^2
		se <- sum(s)
		a <- (y - mean(y))^2
		st <- sum(a)
		fpE <- ((n + p + 1) * se)/(n - p - 1)
		fpe <- ((n + 1) * st)/(n - 1)
		H <- 1 - fpE/fpe
		H
	}
	else {
		if(is.vector(x) == T) {
			n <- length(x)
			p <- 1
			r <- lsfit(x, y)
			s <- r$r^2
			se <- sum(s)
			a <- (y - mean(y))^2
			st <- sum(a)
			fpE <- ((n + p + 1) * se)/(n - p - 1)
			fpe <- ((n + 1) * st)/(n - 1)
			H <- 1 - fpE/fpe
			H
		}
		else {
			n <- nrow(x)
			p <- ncol(x)
			r <- lsfit(x, y)
			s <- r$r^2
			se <- sum(s)
			a <- (y - mean(y))^2
			st <- sum(a)
			fpE <- ((n + p + 1) * se)/(n - p - 1)
			fpe <- ((n + 1) * st)/(n - 1)
			H <- 1 - fpE/fpe
			H
		}
	}
}

aic <- function(x, y)
{
# Using from genshouhou, zoukahou, genzouhou, zougenhou
#                                03/20/00 by Ohmura, Yamahara
#             modified for R     11/08/02 by matsu
	if(is.null(x) == T) {
		n <- length(y)
		p <- 0
		q <- rep(1, n)
		r <- lsfit(q, y, intercept = F)
		s <- r$r^2
		e <- sum(s)
		se <- e/n
		aic <- n * (log(se)) + 2 * (p + 1)
		aic
	}
	else {
		if(is.vector(x) == T) {
			n <- length(x)
			p <- 1
			r <- lsfit(x, y)
			s <- r$r^2
			e <- sum(s)
			se <- e/n
			aic <- n * (log(se)) + 2 * (p + 1)
			aic
		}
		else {
			n <- nrow(x)
			p <- ncol(x)
			r <- lsfit(x, y)
			s <- r$r^2
			e <- sum(s)
			se <- e/n
			aic <- n * (log(se)) + 2 * (p + 1)
			aic
		}
	}
}

FPE <- function(x, y)
{
# Using from genshouhou, zoukahou, genzouhou, zougenhou
#                                03/20/00 by Ohmura, Yamahara
	if(is.null(x) == T) {
		n <- length(y)
		p <- 0
		q <- rep(1, n)
		r <- lsfit(q, y, intercept = F)
		s <- r$r^2
		se <- sum(s)
		fpe <- ((n + p + 1) * se)/(n - p - 1)
		fpe
	}
	else {
		if(is.vector(x) == T) {
			n <- length(x)
			p <- 1
			r <- lsfit(x, y)
			s <- r$r^2
			se <- sum(s)
			fpe <- ((n + p + 1) * se)/(n - p - 1)
			fpe
		}
		else {
			n <- nrow(x)
			p <- ncol(x)
			r <- lsfit(x, y)
			s <- r$r^2
			se <- sum(s)
			fpe <- ((n + p + 1) * se)/(n - p - 1)
			fpe
		}
	}
}
