zougenhou <- function(x, y, method = "adjr2")
{
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                                       <<<<<
#>>>>   Stepwise linear regression procedure (zougen-hou)   <<<<<
#>>>>                                                       <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	imeth <- (match(method, c("adjr2", "adj2r2", "AIC", "FPE", "Cp"), 0))
	if(imeth == 0)
		stop("Invalid method")
	kijun <- function(x0, y, x, imeth)
	{
		if(imeth == 1) {
			kijun <- adjr2(x0, y)
		}
		else if(imeth == 2) {
			kijun <- adj2r2(x0, y)
		}
		else if(imeth == 3) {
			kijun <- -1 * aic(x0, y)
		}
		else if(imeth == 4) {
			kijun <- -1 * FPE(x0, y)
		}
		else if(imeth == 5) {
			kijun <- -1 * Cp(x, y, x0)
		}
		kijun
	}
	if(is.vector(x) == T) {
		Max <- kijun(x, y, x, imeth)
		MaxX <- 1
	}
	else {
		p <- ncol(x)
		bangou <- c(1:p)
		Max <- kijun(x[, 1], y, x, imeth)
		Maxi <- 1
		for(i in c(2:p)) {
			k <- kijun(x[, i], y, x, imeth)
			if(k > Max) {
				Max <- k
				Maxi <- i
			}
		}
		bangou <- bangou[ - Maxi]
		Maxbind <- x[, Maxi]
		z <- ncol(x) - 1
		x1 <- x[,  - Maxi]
		MaxX <- Maxi
		MaxX2 <- Maxi
		while(z > 1) {
			Max2 <- -1e+99
			i <- 1
			while(i <= z) {
				xw <- cbind(Maxbind, x1[, i])
				k <- kijun(xw, y, x, imeth)
				if(k >= Max2) {
				  Max2 <- k
				  MaxX2W <- c(MaxX2, bangou[i])
				  Maxbindw <- xw
				  Maxi2 <- i
				}
				i <- i + 1
			}
			MaxX2 <- MaxX2W
			bangou <- bangou[ - Maxi2]
			Maxbind <- Maxbindw
			l <- ncol(Maxbind)
			m <- 1
			Max3 <- -1e+99
			while(m < l) {
				Maxbindm <- Maxbind[,  - m]
				k2 <- kijun(Maxbindm, y, x, imeth)
				if(k2 >= Max3) {
				  Max3 <- k2
				  i2 <- m
				}
				m <- m + 1
			}
			Maxbindl <- Maxbind[,  - l]
			k3 <- kijun(Maxbindl, y, x, imeth)
			if(Max3 > k3) {
				Max2 <- Max3
				bangou2 <- c(bangou, MaxX2[i2])
				bangou <- bangou2
				MaxX2 <- MaxX2[ - i2]
				x1 <- cbind(x1, Maxbind[, i2])
				Maxbind2 <- Maxbind[,  - i2]
				Maxbind <- Maxbind2
				z <- ncol(x1) - 1
				x1 <- x1[,  - Maxi2]
			}
			else {
				z <- ncol(x1) - 1
				x1 <- x1[,  - Maxi2]
			}
			if(Max2 > Max) {
				Max <- Max2
				MaxX <- MaxX2
			}
		}
		xw <- cbind(Maxbind, x1)
		k <- kijun(xw, y, x, imeth)
		if(k >= Max) {
			Max <- k
			MaxX <- c(MaxX2, bangou)
		}
	}
	teisuu <- kijun(NULL, y, x, imeth)
	if(teisuu > Max) {
		Max <- teisuu
		MaxX <- NA
	}
	if(imeth == 1) {
		ans <- list(adjr2 = Max, kumiawase = MaxX)
	}
	else if(imeth == 2) {
		ans <- list(adj2r2 = Max, kumiawase = MaxX)
	}
	else if(imeth == 3) {
		Max <- -1 * Max
		ans <- list(AIC = Max, kumiawase = MaxX)
	}
	else if(imeth == 4) {
		Max <- -1 * Max
		ans <- list(FPE = Max, kumiawase = MaxX)
	}
	else if(imeth == 5) {
		Max <- -1 * Max
		ans <- list(Cp = Max, kumiawase = MaxX)
	}
	ans	#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#           x: independent variables matrix
#           y: dependent variable
#      method: A criterion among the following list;
#              adjr2: multiple correlation coefficient adjusted for DF
#             adj2r2: multiple correlation coefficient double adjusted for DF
#                AIC: Akaike's information criterion
#                FPE: final prediction error
#                 Cp: Marrows' Cp criterion
#
#  >>> output <<<
# [criterion]: the value of [criterion] on the optimal result
#   kumiawase: numbers of variables on the optimal result
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                                  03/20/01  by  Inoue
#                               modified for R     11/08/02  by  matsu
}

genzouhou <- function(x, y, method = "adjr2")
{
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                                       <<<<<
#>>>>   Stepwise linear regression procedure (genzou-hou)   <<<<<
#>>>>                                                       <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	imeth <- (match(method, c("adjr2", "adj2r2", "AIC", "FPE", "Cp"), 0))
	if(imeth == 0)
		stop("Invalid method")
	kijun <- function(x, y, x0, imeth)
	{
		if(imeth == 1) {
			kijun <- adjr2(x, y)
		}
		else if(imeth == 2) {
			kijun <- adj2r2(x, y)
		}
		else if(imeth == 3) {
			kijun <- -1 * aic(x, y)
		}
		else if(imeth == 4) {
			kijun <- -1 * FPE(x, y)
		}
		else if(imeth == 5) {
			kijun <- -1 * Cp(x0, y, x)
		}
		kijun
	}
	x1 <- x
	if(is.vector(x) == T) {
		MaxMax <- kijun(x, y, x1, imeth)
		Maxi <- 1
	}
	else {
		p <- ncol(x)
		bangou <- c(1:p)
		MaxMax <- kijun(x, y, x1, imeth)
		Maxi <- bangou
		Max <- -1e+99
		for(i in c(1:p)) {
			k <- kijun(x[,  - i], y, x1, imeth)
			if(k > Max) {
				Max <- k
				hazurei <- i
				MaxB <- bangou[ - i]
				xi <- x[, i]
				bangou2 <- i
			}
		}
		z <- ncol(x) - 1
		bangou <- MaxB
		x <- x[,  - hazurei]
		if(Max > MaxMax) {
			MaxMax <- Max
			Maxi <- MaxB
		}
		while(z > 1) {
			i <- 0
			Max <- -1e+99
			while(i < z) {
				i <- i + 1
				B <- x[,  - i]
				k <- kijun(B, y, x1, imeth)
				if(k > Max) {
				  Max <- k
				  MaxB <- bangou[ - i]
				  hazurei <- i
				}
			}
			xi <- cbind(xi, x[, hazurei])
			Maxk2 <- -1e+99
			kari2 <- c(bangou2, bangou[hazurei])
			bangou2 <- kari2
			l <- ncol(xi)
			m <- 1
			while(l > m) {
				x2 <- cbind(x[,  - hazurei], xi[, m])
				k2 <- kijun(x2, y, x1, imeth)
				if(Maxk2 < k2) {
				  n <- m
				  Maxk2 <- k2
				}
				m <- m + 1
			}
			x3 <- cbind(x[,  - hazurei], xi[, l])
			Maxk3 <- kijun(x3, y, x1, imeth)
			if(Maxk3 < Maxk2) {
				x <- cbind(x[,  - hazurei], xi[, n])
				kariB <- c(MaxB, bangou2[n])
				MaxB <- kariB
				bangou2 <- bangou2[ - n]
				Max <- Maxk2
				xi <- xi[,  - n]
			}
			else {
				z <- ncol(x) - 1
				x <- x[,  - hazurei]
			}
			bangou <- MaxB
			if(Max > MaxMax) {
				MaxMax <- Max
				Maxi <- MaxB
			}
		}
	}
	n <- NULL
	teisuu <- kijun(n, y, x1, imeth)
	if(teisuu > MaxMax) {
		MaxMax <- teisuu
		Maxi <- NA
	}
	if(imeth == 1) {
		ans <- list(adjr2 = MaxMax, kumiawase = Maxi)
	}
	else if(imeth == 2) {
		ans <- list(adj2r2 = MaxMax, kumiawase = Maxi)
	}
	else if(imeth == 3) {
		MaxMax <- -1 * MaxMax
		ans <- list(AIC = MaxMax, kumiawase = Maxi)
	}
	else if(imeth == 4) {
		MaxMax <- -1 * MaxMax
		ans <- list(FPE = MaxMax, kumiawase = Maxi)
	}
	else if(imeth == 5) {
		MaxMax <- -1 * MaxMax
		ans <- list(Cp = MaxMax, kumiawase = Maxi)
	}
	ans	#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#           x: independent variables matrix
#           y: dependent variable
#      method: A criterion among the following list;
#              adjr2: multiple correlation coefficient adjusted for DF
#             adj2r2: multiple correlation coefficient double adjusted for DF
#                AIC: Akaike's information criterion
#                FPE: final prediction error
#                 Cp: Marrows' Cp criterion
#
#  >>> output <<<
# [criterion]: the value of [criterion] on the optimal result
#   kumiawase: numbers of variables on the optimal result
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                               03/20/01  by  Kuwayama
#                            modified for R     11/08/02  by  matsu
}

Cp <- function(x, y, a)
{
# Using from genzouhou, zougenhou
#            03/20/01  by  Inoue, kuwayama
	if(is.null(a) == T) {
		n <- length(y)
		p <- ncol(x)
		q <- rep(1, n)
		r <- lsfit(x, y)
		s <- r$r^2
		se <- sum(s)
		ve <- se/(n - p - 1)
		i <- lsfit(q, y, intercept = F)
		j <- i$r^2
		je <- sum(j)
		pa <- 0
		Cp <- je/ve + 2 * pa
		Cp
	}
	else {
		if(is.vector(a) == T) {
			pa <- 1
		}
		else {
			pa <- ncol(a)
		}
		n <- nrow(x)
		p <- ncol(x)
		r <- lsfit(x, y)
		s <- r$r^2
		se <- sum(s)
		ve <- se/(n - p - 1)
		i <- lsfit(a, y)
		j <- i$r^2
		je <- sum(j)
		Cp <- je/ve + 2 * pa
		Cp
	}
}
