modelset <- function( x, y, q=0.05, method="BH"){

#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                                                       <<<<<
#>>>>                  the function to find the MODEL SET                   <<<<<
#>>>>                                                                       <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

  #>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<
  #> all the combinations of explanatory valiables  <<
  #>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<
	
  model.cand <- function(x){
  
    mk.model <- function(model.cand){
      ncand <- length(model.cand)
      model <- NULL
      for(i in 1:ncand){
        model.temp <- model.cand[[i]]
        nval <- length(model.temp)
        for(j in 1:nval){
          model <- c(model, list(model.temp[-j]))
        }
      }
      temp <- NULL
      ncand <- length(model)
      for(i in 1:(ncand-1)){
        for(j in (i+1):ncand){
          if(setequal(model[[i]],model[[j]])==TRUE){
            temp <- c(temp ,j)
          }
        }
      }
      if(is.null(temp) == TRUE){
        model
      }else{
        model[-temp]
      }
	}
    x.col <- ncol(x)
    cand <- list(1:x.col)
    temp <- cand
    for(i in 1:(x.col-1)){
      temp <- mk.model(temp)
      cand <- c(cand, temp)
    }
    output <- c(cand, list(NULL))
    return(output)
  }
  
  #>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<
  #>>>            AIC in the candidates           <<<<
  #>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<
  
  model.aic <- function(x, y){
  
    cand <- model.cand(x)
    n.cand <- length(cand)
    aic.vec <- NULL
    for(i in 1:n.cand){
      if(is.null(cand[[i]])==TRUE){
        temp <- aic(NULL, y)
        aic.vec <- c(aic.vec, temp)
      }else{
        temp <- aic(x[,cand[[i]] ], y)
        aic.vec <- c(aic.vec, temp)
      }
    }
    temp <- order(aic.vec)
    cand <- cand[temp]
    output <- list(candidate=cand, AIC=sort(aic.vec))
    return(output)
  }
  
  #>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<
  #>>>>            log-likelihood              <<<<
  #>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<
  
  likelihood.vec <- function(x, y){
  
    if(is.null(x) == TRUE){
      n <- length(y)
      p <- 0
      x <- rep(1,n)
      res <- lsfit(x, y, intercept=F)$residuals
      sigma2 <- sum(res^2) / n
    }else{
      res <- lsfit(x,y)$residuals
      if(is.vector(x) == TRUE){
        sigma2 <- sum(res^2) / length(x)
      }else{
        sigma2 <- sum(res^2) / nrow(x)
      }
    }
    output <- (res^2) / (-2 * sigma2) - (log(2 * pi * sigma2)) / 2
    return(output)
  }
  
  #>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<
  #>>>>    whether x(y) is contained in y(x)   <<<<
  #>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<
  
  is.imply <- function(x,y){
  
    if(length(x) <= length(y)){
      if(all(is.element(x,y)) == TRUE){
        output <- TRUE
      }else{
        output <- FALSE
      }
    }else{
      if(all(is.element(y,x)) == TRUE){
        output <- TRUE
      }else{
        output <- FALSE
      }
    }
    return(output)
  }
  
  
  
  #>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<
  #>>>>               Linhart test             <<<<
  #>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<
  
  aic.test <- function(x1,x2,y){
  
    n <- length(y)
    L.vec1 <- likelihood.vec(x1,y)
    L.vec2 <- likelihood.vec(x2,y)
    MaxL1 <- sum(L.vec1)
    MaxL2 <- sum(L.vec2)
    Ldif.e <- (MaxL1 - MaxL2)/n
    Ldif.var <- n / (n - 1) * sum(((L.vec1 - L.vec2) - Ldif.e)^2)
    AICdif.var <- 4 * Ldif.var
    z <- (aic(x1, y) - aic(x2, y)) / sqrt(AICdif.var)
    if(z < 0){
      p <- pnorm(z)
      # p <- pt(z, n - 1)
    }else{
      p <- 1 - pnorm(z)
    }
    return(2*p)
  }
  
  #>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<
  #>>>>         likelihood ratio test          <<<<
  #>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<
  
  yudo.test <- function(x1,x2,y){
  
    MaxL1 <- sum(likelihood.vec(x1,y))
    MaxL2 <- sum(likelihood.vec(x2,y))
    if(is.null(x1) == TRUE){
      dim1 <- 0
    }else{
      if(is.vector(x1) == TRUE){
        dim1 <- 1
      }else{
        dim1 <-ncol(x1)
      }
    }
    if(is.null(x2) == TRUE){
      dim2 <- 0
    }else{
      if(is.vector(x2) == TRUE){
        dim2 <- 1
      }else{
        dim2 <-ncol(x2)
      }
    }
    if(dim1 <= dim2){
      p <- 1 - pchisq(2 * (MaxL2 - MaxL1),(dim2 - dim1))
    }else{
      p <- 1 - pchisq(2 * (MaxL1 - MaxL2),(dim1 - dim2))
    }
    return(p)
  }
  
  #>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<
  #>>>             the method of Holm             <<<<
  #>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<
  
  Holm <- function(p, alpha){
  
    temp <- length(p) - 1
    for(i in (temp+1):2){
      if(Pvalue.vec[i] > (alpha/temp)){
        k <- i
        break
      }else{
        temp <- temp - 1
        if( i == 2){
          k <- 1
        }
      }
    }
    return(k)
  }
  
  #>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<
  #>>>              the method of BH              <<<<
  #>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<
  
  BH <- function(p, q){
  
    k <- NULL
    temp <- length(p) - 1
    for(i in 1:temp){
      if(p[i+1] <= ((temp+1-i)/temp*q)){
        k <- i
        break
      }else{
      }
    }
    if(is.null(k) == TRUE){
      k <- length(p)
    }
    return(k)
  }
  
  #>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>main<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  
  num <- match(method, c("Holm", "BH"), 0)
  if(num==0){
    stop("Invalid method")
  }
  temp <- model.aic(x,y)
  cand <- temp$candidate
  n <- length(cand)
  aic.vec <- temp$AIC
  Pvalue.vec <- 1
  if(is.null(cand[[1]]) == TRUE){
    for(i in 2:n){
      p <- yudo.test(NULL, x[,cand[[i]] ], y)
      Pvalue.vec <- c(Pvalue.vec, p)
    }
  }else{
    for(i in 2:n){
      if(is.null(cand[[i]]) == TRUE){
        p <- yudo.test(x[,cand[[1]] ], NULL, y)
      }else{
        if(is.imply(cand[[1]], cand[[i]]) == TRUE){
          p <- yudo.test(x[,cand[[1]] ], x[,cand[[i]] ], y)
        }else{
          p <- aic.test(x[,cand[[1]] ], x[,cand[[i]] ], y)
        }
      }
      Pvalue.vec <- c(Pvalue.vec, p)
    }
  }
  temp <- rev(order(Pvalue.vec))
  cand <- cand[temp]
  aic.vec <- aic.vec[temp]
  Pvalue.vec <- rev(sort(Pvalue.vec))
  if(num==1){
    k <- Holm(Pvalue.vec, q)
  }else if(num==2){
    k <- BH(Pvalue.vec, q)
  }
  score <- numeric(ncol(x))
  for(j in 1:k){
    temp <- numeric(ncol(x))
    temp[cand[[j]]] <- 1
    score <- score + (1/sum(temp)) * temp
  }
  output <- list(model.set=k, score=score, 
    candidate=cand, p.value=Pvalue.vec, AIC=aic.vec)
  return(output)
  
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#    modelset(x, y, q, method)
#      x      : matrix data(explanatory variables)
#      y      : vector data(dependent variable)
#      q      : significance level("Type I FWE" or "FDR")
#      method : the applying method of multiple comparison("Holm" or "BH")
#
#  >>> output <<<
#      model.set : the number of the model(s) included in model set
#      score     : the score of each variables
#      candidate : the candidates in the descending ordered p-value
#      p.value   : the descending ordered p-value
#      AIC       : in the descending ordered p-value
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                                          12/28/07  by Tanahashi
#                                               modified   12/26/08  by matsu
}